#  ooo  oooo  oooo o   o     o    ooo   ooo  o   o  oooo ooooo o  ooo   oooo
# 0   0 0   0 0    00  0    0 0  0   0 0   0 0   0 0       0   0 0   0 0
# 0   0 oooo  oooo 0 0 0   0   0 0     0   0 0   0  oooo   0   0 0      oooo
# 0   0 o     0    0 o 0   ooooo 0   o 0   0 0   0      0  0   0 0   o      0
#  ooo  o     oooo o  0o  0     0 ooo   ooo   ooo  0oooo   0   0  ooo  0oooo
#
#  (  (  (  (  (  (  (  (  (  (  (  (  O  )  )  )  )  )  )  )  )  )  )  )  )
#
#    OpenAcoustics: The Open Source Collection for Computational Acoustics
#
#               Copyright (C) 2008-2016 The OpenAcoustics Team
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


from pylab import *
from scipy import *
#from mpl_toolkits.mplot3d import axes3d
#from matplotlib import cm
import matplotlib.pylab as plt
import matplotlib.path as Path

############## Parameters for calculation #########
dx = 0.01
dy = dx

dt = 0.00002

Ro=1.21
C=343.
K=Ro*C*C

sx = 0.3
sy = 0.4

############## 2D room shape ############
verts = array([
    [-2,  1.5],
    [-2, -1.5],
    [ 2, -1.5],
    [ 2,  1.5],
    [-2,  1.5]
    ])
codes = array([1, 2, 2, 2, 79])

############## making boundary indexes #########
pth = Path.Path(verts, codes)
x_min = min(verts[:,0])
x_max = max(verts[:,0])
y_min = min(verts[:,1])
y_max = max(verts[:,1])

x = np.arange(x_min+0.5*dx, x_max+0.5*dx, dx)
y = np.arange(y_min+0.5*dy, y_max+0.5*dy, dy)

X = len(x)
Y = len(y)

bools = zeros((X,Y), dtype="bool")

for j in range(Y):
    points = [[x[i], y[j]] for i in range(X)]
    bools[:,j] = pth.contains_points(points)

boundary_x_in = []
boundary_x_out = []
for i in range(X-1):
    for j in range(Y):
        if(bools[i,j] == 0 and bools[i+1,j] == 1):
            boundary_x_in.append([i+1,j])
        elif(bools[i,j] == 1 and bools[i+1,j] == 0):
            boundary_x_out.append([i,j])
boundary_x_in = np.array(boundary_x_in)
boundary_x_out = np.array(boundary_x_out)

boundary_y_in = []
boundary_y_out = []
for i in range(X):
    for j in range(Y-1):
        if(bools[i,j] == 0 and bools[i,j+1] == 1):
            boundary_y_in.append([i,j+1])
        elif(bools[i,j] == 1 and bools[i,j+1] == 0):
            boundary_y_out.append([i,j])
boundary_y_in = np.array(boundary_y_in)
boundary_y_out = np.array(boundary_y_out)

######## sound source signal (time domain) #######
ang = arange(-pi,pi,2.*pi/50.)
sig = cos(ang)
sig += 1

ssx = int((sx - x_min)/dx)
ssy = int((sy - y_min)/dy)

######### making arrays for calculation #########
P1 = zeros((X,Y),"float64")
P2 = zeros((X,Y),"float64")

Ux1 = zeros((X+1,Y),"float64")
Ux2 = zeros((X+1,Y),"float64")

Uy1 = zeros((X,Y+1),"float64")
Uy2 = zeros((X,Y+1),"float64")

#Z=Ro*C*20

########## Leap Frog ##########
for t in range(150):
    if t<len(sig):
        P1[ssx,ssy] += sig[t]

    Ux2[1:X,:]=Ux1[1:X,:]-dt/Ro/dx*(P1[1:X,:]-P1[:X-1,:])
    Uy2[:,1:Y]=Uy1[:,1:Y]-dt/Ro/dy*(P1[:,1:Y]-P1[:,:Y-1])

    A=dt/(Ro*dx)
    #Ux2[0,:]=P1[0,:]/-Z
    #Ux2[-1,:]=P1[-1,:]/Z
    for n in range(len(boundary_x_in)):
        Ux2[boundary_x_in[n,0], boundary_x_in[n,1]] = 0.0
    for n in range(len(boundary_x_out)):
        Ux2[boundary_x_out[n,0]+1, boundary_x_out[n,1]] = 0.0
    #Ux2[0,:]=(1./(1.+Z*A))*( (1.-Z*A)*Ux1[0,:]-2*A*P1[1,:])
    #Ux2[-1,:]=(1./(1.+Z*A))*( (1.-Z*A)*Ux1[-1,:]+2*A*P1[-1,:])

    #Uy2[:,0]=P1[:,0]/-Z
    #Uy2[:,-1]=P1[:,-1]/Z
    for n in range(len(boundary_y_in)):
        Uy2[boundary_y_in[n,0], boundary_y_in[n,1]] = 0.0
    for n in range(len(boundary_y_out)):
        Uy2[boundary_y_out[n,0], boundary_y_out[n,1]+1] = 0.0
    #Uy2[:,0]=(1./(1.+Z*A))*( (1.-Z*A)*Uy1[:,0]-2*A*P1[:,0])
    #Uy2[:,-1]=(1./(1.+Z*A))*( (1.-Z*A)*Uy1[:,-1]+2*A*P1[:,-1])

    P2[:X,:Y] = P1[:X,:Y]-K*dt/dx*(Ux2[1:X+1,:]-Ux2[:X,:]) \
                -K*dt/dy*(Uy2[:,1:Y+1]-Uy2[:,:Y])

    P1,P2=P2,P1
    Ux1,Ux2=Ux2,Ux1
    Uy1,Uy2=Uy2,Uy1

###### Draw the room shape and result ######
yy, xx = meshgrid(y,x)
P2masked = ma.array(P2, mask=logical_not(bools))
plt.pcolor(xx, yy, P2masked, cmap=cm.jet)

xs, ys = zip(*verts)
plt.plot(xs, ys, 'o--', lw=2, color='black', ms=10)
plt.axis('equal')
plt.clim(-0.02, 0.02)
plt.colorbar()
plt.ylim([-3.4,3.4])
plt.xlim([-3.4,3.4])
plt.savefig('sampleFDTD_rectangle.png')
plt.show()
