#  ooo  oooo  oooo o   o     o    ooo   ooo  o   o  oooo ooooo o  ooo   oooo
# 0   0 0   0 0    00  0    0 0  0   0 0   0 0   0 0       0   0 0   0 0
# 0   0 oooo  oooo 0 0 0   0   0 0     0   0 0   0  oooo   0   0 0      oooo
# 0   0 o     0    0 o 0   ooooo 0   o 0   0 0   0      0  0   0 0   o      0
#  ooo  o     oooo o  0o  0     0 ooo   ooo   ooo  0oooo   0   0  ooo  0oooo
#
#  (  (  (  (  (  (  (  (  (  (  (  (  O  )  )  )  )  )  )  )  )  )  )  )  )
#
#    OpenAcoustics: The Open Source Collection for Computational Acoustics
#
#               Copyright (C) 2008-2012 The OpenAcoustics Team
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


from pylab import *
from scipy import *
from numpy import cos, arange, zeros, meshgrid
from mpl_toolkits.mplot3d import axes3d
from matplotlib import cm

X=40
Y=30

dx=0.1
dy=0.1
dt=0.0001

Ro=1.21
C=343
K=Ro*C*C

ang = arange(-pi,pi,2*pi/50)
sig = cos(ang)
sig += 1

P = zeros((X,Y),"float64")
Ux = zeros((X+1,Y),"float64")
Uy = zeros((X,Y+1),"float64")

fig = figure()

if matplotlib.__version__ < '3.4':
    ax = axes3d.Axes3D(fig)
else:
    ax = axes3d.Axes3D(fig,auto_add_to_figure=False)
    fig.add_axes(ax)

x = arange(0, dx*X,dx)
y = arange(0, dy*Y,dy)
xx, yy = meshgrid(y,x)
surf = ax.plot_surface(xx,yy,P,rstride=1, cstride=1,cmap=cm.gray)
oldsurf=surf

Z=Ro*C

for t in range(40):
    if t<len(sig):
        P[X//2,Y//2] += sig[t]

    Ux[1:X,:]=Ux[1:X,:]-dt/Ro/dx*(P[1:X,:]-P[:X-1,:])
    Uy[:,1:Y]=Uy[:,1:Y]-dt/Ro/dy*(P[:,1:Y]-P[:,:Y-1])
    
    Ux[0,:]=P[0,:]/-Z
    Ux[-1,:]=P[-1,:]/Z
    
    Uy[:,0]=P[:,0]/-Z
    Uy[:,-1]=P[:,-1]/Z

    P[:X,:Y] = P[:X,:Y]-K*dt/dx*(Ux[1:X+1,:]-Ux[:X,:]) \
                -K*dt/dy*(Uy[:,1:Y+1]-Uy[:,:Y])

    surf = ax.plot_surface(xx,yy,P,rstride=1, cstride=1,cmap=cm.jet)
    ax.set_zlim3d(-1,1)
    ax.collections.remove(oldsurf)
    oldsurf=surf
    pause(0.01)
    draw()
